"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.merge = exports.webpack = exports.isServe = exports.webpackArgsWithDefaults = exports.pluginExternals = exports.sharedExternals = exports.compassPluginConfig = exports.createWebConfig = exports.createElectronRendererConfig = exports.createElectronMainConfig = void 0;
const fs_1 = __importDefault(require("fs"));
const webpack_merge_1 = require("webpack-merge");
const react_refresh_webpack_plugin_1 = __importDefault(require("@pmmmwh/react-refresh-webpack-plugin"));
const duplicate_package_checker_webpack_plugin_1 = __importDefault(require("@cerner/duplicate-package-checker-webpack-plugin"));
const path_1 = __importDefault(require("path"));
const module_1 = require("module");
const webpack_bundle_analyzer_1 = require("webpack-bundle-analyzer");
const webpack_plugin_start_electron_1 = require("./webpack-plugin-start-electron");
const args_1 = require("./args");
const loaders_1 = require("./loaders");
const util_1 = require("./util");
const externals_1 = require("./externals");
Object.defineProperty(exports, "sharedExternals", { enumerable: true, get: function () { return externals_1.sharedExternals; } });
Object.defineProperty(exports, "pluginExternals", { enumerable: true, get: function () { return externals_1.pluginExternals; } });
const webpack_plugin_multicompiler_progress_1 = require("./webpack-plugin-multicompiler-progress");
const mini_css_extract_plugin_1 = __importDefault(require("mini-css-extract-plugin"));
const sharedIgnoreWarnings = [
    /Failed to parse source map.+?ENOENT/,
    /require function is used in a way in which dependencies cannot be statically extracted/,
    /the request of a dependency is an expression/,
    /Module not found.+?(mongo_crypt_v1.(dll|so|dylib)|@mongodb-js\/zstd|aws-crt|gcp-metadata)/,
];
const sharedResolveOptions = (target) => {
    if (typeof target === 'string') {
        target = [target];
    }
    return {
        mainFields: target?.includes('web') || target?.includes('webworker')
            ? [
                'compass:browser',
                'compass:module',
                'compass:main',
                'browser',
                'module',
                'main',
            ]
            : ['compass:module', 'compass:main', 'module', 'main'],
        exportsFields: ['compass:exports', 'exports'],
        extensions: ['.jsx', '.tsx', '.ts', '...'],
        alias: {
            browserslist: false,
            'ampersand-sync': false,
            jose: require.resolve('jose'),
        },
    };
};
function createElectronMainConfig(args) {
    const opts = (0, args_1.webpackArgsWithDefaults)(args, { target: 'electron-main' });
    const namedEntry = (0, util_1.entriesToNamedEntries)(opts.entry);
    const config = {
        entry: namedEntry,
        devtool: opts.devtool,
        output: {
            path: opts.outputPath,
            filename: opts.outputFilename ?? '[name].[contenthash].main.js',
            assetModuleFilename: 'assets/[name].[hash][ext]',
            strictModuleErrorHandling: true,
            strictModuleExceptionHandling: true,
        },
        mode: opts.mode,
        target: opts.target,
        module: {
            rules: [
                (0, loaders_1.sourceMapLoader)(opts),
                (0, loaders_1.javascriptLoader)(opts),
                (0, loaders_1.nodeLoader)(opts),
                (0, loaders_1.resourceLoader)(opts),
                (0, loaders_1.sharedObjectLoader)(opts),
                (0, loaders_1.sourceLoader)(opts),
            ],
        },
        node: false,
        externals: (0, util_1.toCommonJsExternal)(externals_1.sharedExternals),
        resolve: {
            aliasFields: [],
            ...sharedResolveOptions(opts.target),
        },
        plugins: [new webpack_plugin_multicompiler_progress_1.WebpackPluginMulticompilerProgress()],
        ignoreWarnings: sharedIgnoreWarnings,
    };
    return (0, webpack_merge_1.merge)(config, opts.mode === 'development'
        ? {
            output: {
                filename: opts.outputFilename ?? '[name].main.js',
                assetModuleFilename: 'assets/[name][ext]',
            },
        }
        : {}, (0, args_1.isServe)(opts) ? { plugins: [new webpack_plugin_start_electron_1.WebpackPluginStartElectron()] } : {}, opts.analyze
        ? {
            plugins: [
                new webpack_bundle_analyzer_1.BundleAnalyzerPlugin({
                    logLevel: 'silent',
                    analyzerPort: 'auto',
                }),
                new duplicate_package_checker_webpack_plugin_1.default(),
            ],
        }
        : {});
}
exports.createElectronMainConfig = createElectronMainConfig;
function createElectronRendererConfig(args) {
    const opts = (0, args_1.webpackArgsWithDefaults)(args, { target: 'electron-renderer' });
    const entries = (0, util_1.entriesToNamedEntries)(opts.entry);
    const config = {
        entry: entries,
        devtool: opts.devtool,
        output: {
            path: opts.outputPath,
            filename: opts.outputFilename ?? '[name].[contenthash].renderer.js',
            assetModuleFilename: 'assets/[name].[hash][ext]',
            library: opts.library ?? (0, util_1.getLibraryNameFromCwd)(opts.cwd),
            libraryTarget: 'umd',
            strictModuleErrorHandling: true,
            strictModuleExceptionHandling: true,
            globalObject: 'globalThis',
        },
        mode: opts.mode,
        target: opts.target,
        module: {
            rules: [
                (0, loaders_1.sourceMapLoader)(opts),
                (0, loaders_1.javascriptLoader)(opts),
                (0, loaders_1.nodeLoader)(opts),
                (0, loaders_1.cssLoader)(opts),
                (0, loaders_1.lessLoader)(opts),
                (0, loaders_1.assetsLoader)(opts),
                (0, loaders_1.sharedObjectLoader)(opts),
                (0, loaders_1.sourceLoader)(opts),
            ],
        },
        plugins: [
            ...(0, util_1.entriesToHtml)(entries),
            new webpack_plugin_multicompiler_progress_1.WebpackPluginMulticompilerProgress(),
        ],
        node: false,
        externals: (0, util_1.toCommonJsExternal)(externals_1.sharedExternals),
        resolve: {
            aliasFields: [],
            ...sharedResolveOptions(opts.target),
        },
        ignoreWarnings: sharedIgnoreWarnings,
    };
    return (0, webpack_merge_1.merge)(config, opts.mode === 'development'
        ? {
            output: {
                filename: opts.outputFilename ?? '[name].renderer.js',
                assetModuleFilename: 'assets/[name][ext]',
            },
        }
        : {}, opts.mode === 'production'
        ? {
            plugins: [
                new mini_css_extract_plugin_1.default(),
            ],
        }
        : {}, (0, args_1.isServe)(opts)
        ? {
            devServer: {
                magicHtml: false,
                port: opts.devServerPort,
                devMiddleware: {
                    writeToDisk: true,
                },
                client: {
                    overlay: {
                        errors: true,
                        warnings: false,
                    },
                },
                https: false,
                hot: opts.hot,
            },
            plugins: [
                new webpack_plugin_start_electron_1.WebpackPluginStartElectron(),
            ].concat(opts.hot
                ? [
                    new react_refresh_webpack_plugin_1.default(),
                ]
                : []),
        }
        : {}, opts.analyze
        ? {
            plugins: [
                new webpack_bundle_analyzer_1.BundleAnalyzerPlugin({
                    logLevel: 'silent',
                    analyzerPort: 'auto',
                }),
                new duplicate_package_checker_webpack_plugin_1.default(),
            ],
        }
        : {});
}
exports.createElectronRendererConfig = createElectronRendererConfig;
function createWebConfig(args) {
    const opts = (0, args_1.webpackArgsWithDefaults)(args, { target: 'web' });
    const { peerDependencies } = require(path_1.default.join(opts.cwd, 'package.json'));
    return {
        entry: (0, util_1.entriesToNamedEntries)(opts.entry),
        devtool: opts.devtool,
        output: {
            path: opts.outputPath,
            filename: opts.outputFilename ?? '[name].js',
            assetModuleFilename: 'assets/[name][ext]',
            library: opts.library ?? (0, util_1.getLibraryNameFromCwd)(opts.cwd),
            libraryTarget: 'umd',
            strictModuleErrorHandling: true,
            strictModuleExceptionHandling: true,
            globalObject: 'globalThis',
        },
        mode: opts.mode,
        target: opts.target,
        module: {
            rules: [
                (0, loaders_1.sourceMapLoader)(opts),
                (0, loaders_1.javascriptLoader)(opts, true),
                (0, loaders_1.nodeLoader)(opts),
                (0, loaders_1.cssLoader)(opts, true),
                (0, loaders_1.lessLoader)(opts),
                (0, loaders_1.assetsLoader)(opts),
                (0, loaders_1.sourceLoader)(opts),
            ],
        },
        externals: {
            ...(0, util_1.toCommonJsExternal)(externals_1.sharedExternals),
            ...(0, util_1.toCommonJsExternal)(Object.keys(peerDependencies ?? {})),
            ...(0, util_1.toCommonJsExternal)(module_1.builtinModules.flatMap((m) => [m, `node:${m}`])),
        },
        resolve: {
            ...sharedResolveOptions(opts.target),
        },
        ignoreWarnings: sharedIgnoreWarnings,
    };
}
exports.createWebConfig = createWebConfig;
function compassPluginConfig(_env, _args) {
    const args = (0, args_1.webpackArgsWithDefaults)(_args);
    const opts = { ...args, hot: true };
    process.env.NODE_ENV = opts.nodeEnv;
    if ((0, args_1.isServe)(opts)) {
        const sandboxMain = path_1.default.join(opts.cwd, 'electron', 'index.js');
        const sandboxRenderer = path_1.default.join(opts.cwd, 'electron', 'renderer', 'index.js');
        try {
            fs_1.default.statSync(sandboxMain);
            fs_1.default.statSync(sandboxRenderer);
        }
        catch (e) {
            throw new Error(`Compass plugin is missing sandbox entry points. To be able to run the plugin in a sandbox outside of Compass, please add ./electron/index.ts and ./electron/renderer/index.ts entry points`);
        }
        return [
            (0, webpack_merge_1.merge)(createElectronMainConfig({
                ...opts,
                entry: sandboxMain,
            }), { externals: (0, util_1.toCommonJsExternal)(externals_1.pluginExternals) }),
            (0, webpack_merge_1.merge)(createElectronRendererConfig({
                ...opts,
                entry: sandboxRenderer,
            }), { externals: (0, util_1.toCommonJsExternal)(externals_1.pluginExternals) }),
        ];
    }
    const entry = fs_1.default.existsSync(path_1.default.join(opts.cwd, 'src', 'index.ts'))
        ? path_1.default.join(opts.cwd, 'src', 'index.ts')
        : path_1.default.join(opts.cwd, 'src', 'index.js');
    return [
        (0, webpack_merge_1.merge)(createElectronRendererConfig({
            ...opts,
            entry,
            outputFilename: 'index.js',
        }), {
            externals: (0, util_1.toCommonJsExternal)(externals_1.pluginExternals),
            plugins: [
                function (compiler) {
                    compiler.hooks.initialize.tap('CleanDistPlugin', () => {
                        try {
                            fs_1.default.rmSync(compiler.outputPath, {
                                recursive: true,
                                force: true,
                            });
                        }
                        catch {
                        }
                    });
                },
            ],
        }),
        (0, webpack_merge_1.merge)(createWebConfig({
            ...opts,
            entry,
            outputFilename: 'browser.js',
        }), {
            externals: (0, util_1.toCommonJsExternal)(externals_1.pluginExternals),
        }),
    ];
}
exports.compassPluginConfig = compassPluginConfig;
var args_2 = require("./args");
Object.defineProperty(exports, "webpackArgsWithDefaults", { enumerable: true, get: function () { return args_2.webpackArgsWithDefaults; } });
Object.defineProperty(exports, "isServe", { enumerable: true, get: function () { return args_2.isServe; } });
var webpack_1 = require("webpack");
Object.defineProperty(exports, "webpack", { enumerable: true, get: function () { return __importDefault(webpack_1).default; } });
var webpack_merge_2 = require("webpack-merge");
Object.defineProperty(exports, "merge", { enumerable: true, get: function () { return webpack_merge_2.merge; } });
//# sourceMappingURL=index.js.map